Attribute VB_Name = "modDCB"
Option Explicit

' The fourteen actual DCB bit-sized data fields within the four
' bytes of fBitFields can be manipulated by bitwise logical And/Or
' operations.
' ----------------------------------------------------------------
Public Const F_BINARY As Long = &H1  ' binary mode, no EOF check
Public Const F_PARITY As Long = &H2  ' enable parity checking
Public Const F_OUT_X_CTS_FLOW As Long = &H4  ' CTS output flow control
Public Const F_OUT_X_DSR_FLOW As Long = &H8  ' DSR output flow control
Public Const F_DTR_CONTROL As Long = &H30  ' DTR flow control type (2 bits)
    Public Const DTR_CONTROL_DISABLE As Long = &H0
    Public Const DTR_CONTROL_ENABLE As Long = &H10
    Public Const DTR_CONTROL_HANDSHAKE As Long = &H20
Public Const F_DSR_SENSITIVITY As Long = &H40  ' DSR sensitivity
Public Const F_TX_CONTINUE As Long = &H80  ' XOFF continues Tx
Public Const F_OUT_X As Long = &H100  ' XON/XOFF out flow control
Public Const F_IN_X As Long = &H200  ' XON/XOFF in flow control
Public Const F_ERROR_CHAR As Long = &H400  ' enable error replacement
Public Const F_NULL As Long = &H800  ' enable null stripping
Public Const F_RTS_CONTROL As Long = &H3000  ' RTS flow control (2 bits)
    Public Const RTS_CONTROL_DISABLE As Long = &H0
    Public Const RTS_CONTROL_ENABLE As Long = &H1000
    Public Const RTS_CONTROL_HANDSHAKE As Long = &H2000
    Public Const RTS_CONTROL_TOGGLE As Long = &H3000
Public Const F_ABORT_ON_ERROR As Long = &H4000  ' abort reads/writes on error
  
Public Type DCB
        DCBlength As Long
        BaudRate As Long
        fBitFields As Long 'See Comments Above
        wReserved As Integer
        XonLim As Integer
        XoffLim As Integer
        ByteSize As Byte
        Parity As Byte
        StopBits As Byte
        XonChar As Byte
        XoffChar As Byte
        ErrorChar As Byte
        EofChar As Byte
        EvtChar As Byte
        wReserved1 As Integer 'Reserved; Do Not Use
End Type

Public Declare Function GetCommState Lib "kernel32" (ByVal hFile As Long, lpDCB As DCB) As Long
Public Declare Function SetCommState Lib "kernel32" (ByVal hFile As Long, lpDCB As DCB) As Long


'************************************************************************************
'   Functions for use with DCB
'
' Use to set the state of DTR
'  Parameters:
'    BitField -> fBitFields member of a DCB
'    DTRValue -> Any one of:
'                       DTR_CONTROL_DISABLE
'                       DTR_CONTROL_ENABLE
'                       DTR_CONTROL_HANDSHAKE
'  Returns:
'    BitField is modified according to the value in DTRValue
'
Public Sub BitFieldSetDTR(ByRef BitField As Long, ByVal DTRValue As Long)
    BitField = (BitField And Not F_DTR_CONTROL) Or DTRValue
End Sub

' Use to set the state of RTS
'  Parameters:
'    BitField -> fBitFields member of a DCB
'    RTSValue -> Any one of:
'                       RTS_CONTROL_DISABLE
'                       RTS_CONTROL_ENABLE
'                       RTS_CONTROL_HANDSHAKE
'                       RTS_CONTROL_TOGGLE
'  Returns:
'    BitField is modified according to the value in RTSValue
'
Public Sub BitFieldSetRTS(ByRef BitField As Long, ByVal RTSValue As Long)
    BitField = (BitField And Not F_RTS_CONTROL) Or RTSValue
End Sub

' Use to retrieve the state of any of the fields in a DCB
'  Parameters:
'    BitField -> fBitFields member of a DCB
'    Field    -> Any one of:
'                       F_BINARY
'                       F_PARITY
'                       F_OUT_X_CTS_FLOW
'                       F_OUT_X_DSR_FLOW
'                       F_DTR_CONTROL
'                       F_DSR_SENSITIVITY
'                       F_TX_CONTINUE
'                       F_OUT_X
'                       F_IN_X
'                       F_ERROR_CHAR
'                       F_NULL
'                       F_RTS_CONTROL
'                       F_ABORT_ON_ERROR
'  Returns:
'   State of field on success (if return is 0, it is off, otherwise
'   its equal to Field)
'   -1 otherwise
'
Public Function BitFieldGet(ByVal BitField As Long, ByVal Field As Long) As Long
    If (Field <> F_BINARY And Field <> F_PARITY And Field <> F_OUT_X_CTS_FLOW And _
        Field <> F_OUT_X_DSR_FLOW And Field <> F_DTR_CONTROL And _
        Field <> F_DSR_SENSITIVITY And Field <> F_TX_CONTINUE And Field <> F_OUT_X And _
        Field <> F_IN_X And Field <> F_ERROR_CHAR And Field <> F_NULL And _
        Field <> F_RTS_CONTROL And Field <> F_ABORT_ON_ERROR) Then
        
        BitFieldGet = -1
    Else
        BitFieldGet = BitField And Field
    End If
End Function

' Use to set a field's bit to on (1)
'  Parameters:
'    BitField -> fBitFields member of a DCB
'    Field    -> Any one of:
'                       F_BINARY
'                       F_PARITY
'                       F_OUT_X_CTS_FLOW
'                       F_OUT_X_DSR_FLOW
'                       F_DSR_SENSITIVITY
'                       F_TX_CONTINUE
'                       F_OUT_X
'                       F_IN_X
'                       F_ERROR_CHAR
'                       F_NULL
'                       F_ABORT_ON_ERROR
'  Returns:
'    BitField is modified such that the bit specified
'    by Field is turned on (set to 1)
'
'  NOTE:
'    Use BitFieldSetDTR in order to set the value of F_DTR_CONTROL
'    Use BitFieldSetRTS in order to set the value of F_RTS_CONTROL
'
Public Sub BitFieldSetOn(ByRef BitField As Long, ByVal Field As Long)
    If (Field = F_DTR_CONTROL Or Field = DTR_CONTROL_DISABLE Or Field = DTR_CONTROL_ENABLE Or Field = DTR_CONTROL_HANDSHAKE) Then
        MsgBox "Use function BitFieldSetDTR", vbCritical + vbOKOnly, "BitFieldSetOn function..."
    ElseIf (Field = F_RTS_CONTROL Or Field = RTS_CONTROL_DISABLE Or Field = RTS_CONTROL_ENABLE Or Field = RTS_CONTROL_HANDSHAKE Or Field = RTS_CONTROL_TOGGLE) Then
        MsgBox "Use function BitFieldSetRTS", vbCritical + vbOKOnly, "BitFieldSetOn function..."
    Else
        BitField = BitField Or Field
    End If
End Sub

' Use to set a field's bit to off (0)
'  Parameters:
'    BitField -> fBitFields member of a DCB
'    Field    -> Any one of:
'                       F_BINARY
'                       F_PARITY
'                       F_OUT_X_CTS_FLOW
'                       F_OUT_X_DSR_FLOW
'                       F_DSR_SENSITIVITY
'                       F_TX_CONTINUE
'                       F_OUT_X
'                       F_IN_X
'                       F_ERROR_CHAR
'                       F_NULL
'                       F_ABORT_ON_ERROR
'  Returns:
'    BitField is modified such that the bit specified
'    by Field is turned off (set to 0)
'
'  NOTE:
'    Use BitFieldSetDTR in order to set the value of F_DTR_CONTROL
'    Use BitFieldSetRTS in order to set the value of F_RTS_CONTROL
'
Public Sub BitFieldSetOff(ByRef BitField As Long, ByVal Field As Long)
    If (Field = F_DTR_CONTROL Or Field = DTR_CONTROL_DISABLE Or Field = DTR_CONTROL_ENABLE Or Field = DTR_CONTROL_HANDSHAKE) Then
        MsgBox "Use function BitFieldSetDTR", vbCritical + vbOKOnly, "BitFieldSetOff function..."
    ElseIf (Field = F_RTS_CONTROL Or Field = RTS_CONTROL_DISABLE Or Field = RTS_CONTROL_ENABLE Or Field = RTS_CONTROL_HANDSHAKE Or Field = RTS_CONTROL_TOGGLE) Then
        MsgBox "Use function BitFieldSetRTS", vbCritical + vbOKOnly, "BitFieldSetOn function..."
    Else
        BitField = BitField And Not Field
    End If
End Sub
'************************************************************************************


